package com.perforce.p4simulink.connection;

/**
 * Perforce SCM Integration
 */

import com.perforce.p4simulink.P4CMException;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.io.ByteArrayInputStream;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.Collection;
import java.util.HashSet;
import java.util.Properties;

public class Config {
    private static final Logger log        = LogManager.getLogger(Config.class.getName());
    private static final String[] P4VARS   = {"P4PORT", "P4CLIENT", "P4USER"};
    private static final String P4CONFIG   = System.getenv("P4CONFIG") != null ? System.getenv("P4CONFIG") : ".p4rc";
    private static final String P4SIMULINK = ".p4simulink";

    private Path path = null;
    private Properties properties = new Properties();

    /**
     * Check if a config file named P4CONFIG or P4SIMULINK exists in the specified path.
     *
     * @param path Local file path to check for a Perforce config file.
     * @return true if a config file was found in the named path, otherwise false.
     */
    public boolean exists(Path path) {
        Path configPath = null;
        Path simulinkPath = path.resolve(P4SIMULINK);
        if (P4CONFIG != null) {
            configPath = path.resolve(P4CONFIG);
        }

        if (configPath != null && Files.exists(configPath)) {
            this.path = configPath;
            log.trace("Found config file {}", configPath.toString());
            return true;
        } else if (Files.exists(simulinkPath)) {
            this.path = simulinkPath;
            log.trace("Found config file {}", simulinkPath);
            return true;
        } else {
            return false;
        }
    }

    /**
     * Load the config file into memory, making sure that at least P4PORT, P4USER and P4CLIENT are set.
     *
     * @throws com.perforce.p4simulink.P4CMException
     */
    public void load() throws P4CMException {
    	try (FileInputStream in = new FileInputStream(path.toFile())) {
    		load(in);
        } catch (IOException e) {
            throw new P4CMException("Error loading config file " + path.toString(), e);
        }
    }
    
    public void load(String string) throws P4CMException {
    	load(new ByteArrayInputStream(string.getBytes()));
    }

    public void load(InputStream is) throws P4CMException {
    	properties = new Properties();
        try {
            properties.load(is);
            if (properties == null) {
                throw new P4CMException("Invalid config file.");
            }
            for (String var : P4VARS) {
                if (properties.getProperty(var) == null) {
                    throw new P4CMException("Missing required key " + var);
                }
            }
            log.trace("Perforce configuration loaded: {}", properties.toString());
        } catch (IOException e) {
            throw new P4CMException("Error loading config file " + path.toString(), e);
        }
    }
    
    /**
     * Return a collection of the config names that P4Config could be working with.
     *
     * @return List of config file names.
     */
    public Collection<String> getConfigNames() {
        Collection<String> configNames = new HashSet<>();
        configNames.add(P4SIMULINK);
        if (P4CONFIG != null) {
            configNames.add(P4CONFIG);
        }
        log.trace("Config File Names: {}", configNames.toString());
        return configNames;
    }

    /**
     * Extracts P4PORT from the loaded config and returns a P4Java suitable URI.
     *
     * @return P4PORT converted to a P4Java URI
     * @throws P4CMException
     */
    public String getAddress() throws P4CMException {
        String port = properties.getProperty("P4PORT");
        String[] parts = port.split(":", 2);
        if (parts.length != 2) {
            throw new P4CMException("Invalid P4PORT: " + port);
        }

        StringBuilder address = new StringBuilder();
        if (parts[0].startsWith("ssl")) {
            address.append("p4javassl://");
            address.append(parts[1]);
        } else {
            address.append("p4java://");
            address.append(port);
        }
        log.trace("Perforce Server: {}", address.toString());
        return address.toString();
    }

    public String getPort() throws P4CMException {
    	String address = getAddress();
    	if (address.lastIndexOf(":") > 0) {
    		return address.substring(address.lastIndexOf(":"));
    	}
    	return "";
    }
    
    public String getClient() {
        return properties.getProperty("P4CLIENT");
    }

    public String getUser() {
        return properties.getProperty("P4USER");
    }

    public void setAddress(String address) {
    	properties.setProperty("P4PORT", address);
    }
    
    public void setClient(String client) {
    	properties.setProperty("P4CLIENT", client);
    }
    
    public void setUser(String user) {
    	properties.setProperty("P4USER", user);
    }
    
    @Override
    public String toString() {
        String port = properties.getProperty("P4PORT");
        String user = properties.getProperty("P4USER");
        String client = properties.getProperty("P4CLIENT");
        return "CLIENT: '" + client + "' USER: '" + user + "' PORT: '" + port + "'";
    }
}
